##
# This file uses the R code to estimate the size of the ensemble using the
# method proposed in:
#
# Hernández-Lobato, D.; Martínez-Muñoz, G. & Suárez, A. 
# How large should ensembles of classifiers be? Pattern Recognition, 2013, 46, 1323 - 1336
#

library(mlbench)
source("rf_size.R")

set.seed(0)

# We will use the pima dataset

data(PimaIndiansDiabetes)
data <- PimaIndiansDiabetes

# We randomly split the data into train and test

i_train <- sample(1 : nrow(data), round(2 / 3 * nrow(data)))

train <- data[ i_train, ]
test <- data[ -i_train, ]

# We create a random forest with the adequate ensemble size

result <- create_rf_adequate_size(diabetes~., train)

rf_adequate_size <- result$rf
generated_trees <- result$total_generated_trees

# We now create a random forest that will serve as a proxy for the asymptotic ensemble

rf_asymptotic <- create_random_forest(diabetes~., train, 1e4)

pred_rf_adequate_size <- error_and_classes(rf_adequate_size, test)
pred_rf_asymptotic <- error_and_classes(rf_asymptotic, test)

cat("Pima Indian Diabetes:\n")
cat("\tEstimated ensemble size:", rf_adequate_size$ntree, "\n")
cat("\tGenerated trees in practice:", generated_trees, "\n")
cat("\tTest disagreement rate in class predictions with respect to the asymptotic ensemble:", 
	mean(pred_rf_asymptotic$predictions != pred_rf_adequate_size$predictions),  "\n")
cat("\tGenerated ensemble test error:", pred_rf_adequate_size$error, "\n")
cat("\tAsymptotic ensemble test error:", pred_rf_asymptotic$error, "\n")

# We will use the breast cancer dataset

data(BreastCancer)
data <- BreastCancer

# We discard the sample ID which is useless for prediction

data <- data[, -1 ]

for (i in 1 : ncol(data))
	if (is.ordered(data[, i ]))
		data[ , i ] <- as.numeric(data[, i ])

# We randomly split the data into train and test

i_train <- sample(1 : nrow(data), round(2 / 3 * nrow(data)))

train <- data[ i_train, ]
test <- data[ -i_train, ]

# We create a random forest with the adequate ensemble size

result <- create_rf_adequate_size(Class~., train)

rf_adequate_size <- result$rf
generated_trees <- result$total_generated_trees

# We now create a random forest that will serve as a proxy for the asymptotic ensemble

rf_asymptotic <- create_random_forest(Class~., train, 1e4)

pred_rf_adequate_size <- error_and_classes(rf_adequate_size, test)
pred_rf_asymptotic <- error_and_classes(rf_asymptotic, test)

cat("Breast Cancer:\n")
cat("\tEstimated ensemble size:", rf_adequate_size$ntree, "\n")
cat("\tGenerated trees in practice:", generated_trees, "\n")
cat("\tTest disagreement rate in class predictions with respect to the asymptotic ensemble:", 
	mean(pred_rf_asymptotic$predictions != pred_rf_adequate_size$predictions),  "\n")
cat("\tGenerated ensemble test Error:", pred_rf_adequate_size$error, "\n")
cat("\tAsymptotic ensemble test Error:", pred_rf_asymptotic$error, "\n")





