
library(randomForest)
library(nnet)

##
# create_random_forest 
#
# formula: A formula that describes the model to create eg y~.
# data : data used to train
# n      : number of classifiers to use
#
# Returns: created random forest

create_random_forest <- function(formula, data, n = 100) {
	randomForest(formula, data, ntree = n, na.action = na.roughfix, keep.inbag = TRUE)
}

##
# error_and_classes
# 
# rf: random forest to be evaluated
# data: data used to predict
#
# Returns: predictions of the ensemble and error
#

error_and_classes <- function(rf, data) {

	classes <- levels(data[,ncol(data)])
	target <- data[,ncol(data)]
	votes <- matrix(0, nrow = nrow(data), ncol = length(classes))
	N <- nrow(data)
	M <- rf$ntree

	predictions <- predict(rf, na.roughfix(data), type = "class", predict.all = T, na.action = na.roughfix)$individual

	for (i in 1 : M) {

		if (M > 10) 
			if (i %% round(M / 10) == 0) cat("#")

		indx <- cbind(1 : N, as.integer(factor(predictions[ , i ], levels = classes)))
		votes[ indx ] <- votes[ indx ] + 1
	}

	cat("\n")
	
	error <- mean(factor(classes[ apply(votes, 1, which.is.max) ], levels = classes) != target)
	predictions <- factor(classes[ apply(votes, 1, which.is.max) ], levels = classes)

	list(error = error, votes = votes, predictions = predictions)
}



##
# estimate_adequate_size_oob
# 
# rf : Partially created random forest ensemble 
# data: data used for training the random forest
#
# Returns: an estimate of the adquate size of the ensemble
#

estimate_adequate_size_oob <- function(data, rf) {

	# We estimate the probability that each instances is assigned a particular class label
	
	p <- find_p(rf, data)

	p[ p < 0.5 ] <- 1 - p[ p < 0.5 ]

	binary_search <- function() {

	        evaluate <- function(t) mean(pbeta(p, floor((2 * t + 1) / 2) + 1, 2 * t + 1 - floor((2 * t + 1) / 2))) - 0.99
	        converged <- FALSE

	        upper <- 1e7 + 1
	        lower <- 0

	        while(abs(upper - lower) != 1) {

	                middle <- round((upper - lower) / 2 + lower)
	                v_upper <- evaluate(upper)
	                v_middle <- evaluate(middle)
	                v_lower <- evaluate(lower)
	
	                if(v_middle > 0)
	                        upper <- middle
	                else
	                        lower <- middle
	        }

	        2 * upper + 1
	}

	# We carry out a binary search procedure to find the optimal ensemble size with confidence 99% on the asymptotic
	# class predictions. We employ only odd ensemble sizes. 

	binary_search()
}

##
# find_p
# 
# rf : random forest ensemble
# data : data used to generate the random forest
#
# Note : asumes the last column of data has the class labels 
#
# returns: a vector with the probability that each instance receives a particular class label
#

find_p <- function(rf, data) {

	classes <- levels(data[, ncol(data) ])
	target <- data[, ncol(data) ]
	N <- nrow(data)
	M <- rf$ntree

	predictions <- predict(rf, na.roughfix(data), type = "class", predict.all = T, na.action = na.roughfix)$individual[, 1 : M ]

	inbag <- rf$inbag
	p <- 1 : nrow(data)
	
	for (i in 1 : nrow(predictions)) {
		sel <- which(inbag[ i, ] == 0)
		p[ i ] <- mean(predictions[ i, sel ] == classes[ 2 ])
	}

	p
}


##
# create_rf_adequate_size
#
# This function creates the random forest ensemble with a size that will give with 99% prob. the asymptotic predictions
#
# formula: formula used to create the random forest
# data : data to use for training
# initial_seed : random seed used in the process of generating the tree 
#
# Devuelve: calculada
#

create_rf_adequate_size <- function(formula, data, initial_seed = 0) {

	# The initial ensemble size is 100

	n_trees <- 100

	while(TRUE) {

		# The following two lines are equivalent to adding (n_trees - rf$ntree) trees to the random forest 

		set.seed(initial_seed)
		rf <- create_random_forest(formula, data, n = n_trees) 

		size <- estimate_adequate_size_oob(data, rf)

		if (size <= n_trees) {
			generated <- n_trees
			break
		}

		n_trees <- min(size, 2 * n_trees)
	}

	# We only keep the generated number of trees

	rf$ntree <- size

	# We return a list with random forest ensemble and the number of actual trees generated

	list(rf = rf, total_generated_trees = generated)
}


